==> grafana-alloy-debug-1.10.2-1-x86_64.pkg.tar.zst: usr/src/debug/grafana-alloy/pkg/mod/github.com/!data!dog/opentelemetry-mapping-go/pkg/inframetadata@v0.28.0/internal/hostmap/tags.go <==
// Unless explicitly stated otherwise all files in this repository are licensed
// under the Apache License Version 2.0.
// This product includes software developed at Datadog (https://www.datadoghq.com/).
// Copyright 2023-present Datadog, Inc.

package hostmap

import (
	"errors"
	"fmt"
	"sort"
	"strings"

	"go.opentelemetry.io/collector/pdata/pcommon"
	conventions "go.opentelemetry.io/collector/semconv/v1.21.0"
)

const (
	hostTagPrefix      = "datadog.host.tag."
	hostAliasAttribute = "datadog.host.aliases"
)

var hostTagMapping = map[string]string{
	conventions.AttributeDeploymentEnvironment: "env",
	conventions.AttributeK8SClusterName:        "cluster_name",
	conventions.AttributeCloudProvider:         "cloud_provider",
	conventions.AttributeCloudRegion:           "region",
	conventions.AttributeCloudAvailabilityZone: "zone",

	// TODO(OTEL-1766): import of semconv 1.27.0 is blocked on Go1.22 support
	"deployment.environment.name": "env",
}

// assertStringValue returns the string value of the given value, or an error if the value is not a string.
func assertStringValue(name string, v pcommon.Value) (string, error) {
	if v.Type() != pcommon.ValueTypeStr {
		return "", mismatchErr(name, v.Type(), pcommon.ValueTypeStr)
	}
	return v.Str(), nil
}

// getHostTags returns a slice of tags from the given map.
func getHostTags(m pcommon.Map) ([]string, error) {
	var tags []string
	var err error
	m.Range(func(k string, v pcommon.Value) bool {
		if strings.HasPrefix(k, hostTagPrefix) { // User-defined tags
			if str, err2 := assertStringValue(k, v); err2 != nil {
				err = errors.Join(err, err2)
			} else if str == "" {
				err = errors.Join(err, fmt.Errorf("attribute %q has empty string value, expected non-empty string", k))
			} else {
				tags = append(tags, k[len(hostTagPrefix):]+":"+str)
			}
		} else if mappedKey, ok := hostTagMapping[k]; ok { // Well-known tags
			if str, err2 := assertStringValue(k, v); err2 != nil {
				err = errors.Join(err, err2)
			} else {
				tags = append(tags, mappedKey+":"+str)
			}
		}
		return true
	})

	// Allow for comparison of tags
	sort.Strings(tags)
	return tags, err
}

// getHostAliases tries to get a host aliases from attribute datadog.host.aliases
func getHostAliases(attrs pcommon.Map) []string {
	var hostAliases []string
	attrs.Range(func(k string, v pcommon.Value) bool {
		if k == hostAliasAttribute {
			if v.Type() != pcommon.ValueTypeSlice {
				return false
			}
			hostAliasesAny := v.Slice().AsRaw()
			for _, hostAlias := range hostAliasesAny {
				if hostAliasStr, ok := hostAlias.(string); ok {
					hostAliases = append(hostAliases, hostAliasStr)
				}
			}
			return false
		}
		return true
	})
	return hostAliases
}

==> grafana-alloy-debug-1.11.2-1-x86_64.pkg.tar.zst: usr/src/debug/grafana-alloy/pkg/mod/github.com/!data!dog/opentelemetry-mapping-go/pkg/inframetadata@v0.29.1/internal/hostmap/tags.go <==
// Unless explicitly stated otherwise all files in this repository are licensed
// under the Apache License Version 2.0.
// This product includes software developed at Datadog (https://www.datadoghq.com/).
// Copyright 2023-present Datadog, Inc.

package hostmap

import (
	"errors"
	"fmt"
	"sort"
	"strings"

	"go.opentelemetry.io/collector/pdata/pcommon"
	semconv121 "go.opentelemetry.io/otel/semconv/v1.21.0"
	semconv127 "go.opentelemetry.io/otel/semconv/v1.27.0"
)

const (
	hostTagPrefix      = "datadog.host.tag."
	hostAliasAttribute = "datadog.host.aliases"
)

var hostTagMapping = map[string]string{
	string(semconv121.DeploymentEnvironmentKey):     "env",
	string(semconv127.DeploymentEnvironmentNameKey): "env",
	string(semconv127.K8SClusterNameKey):            "cluster_name",
	string(semconv127.CloudProviderKey):             "cloud_provider",
	string(semconv127.CloudRegionKey):               "region",
	string(semconv127.CloudAvailabilityZoneKey):     "zone",
}

// assertStringValue returns the string value of the given value, or an error if the value is not a string.
func assertStringValue(name string, v pcommon.Value) (string, error) {
	if v.Type() != pcommon.ValueTypeStr {
		return "", mismatchErr(name, v.Type(), pcommon.ValueTypeStr)
	}
	return v.Str(), nil
}

// getHostTags returns a slice of tags from the given map.
func getHostTags(m pcommon.Map) ([]string, error) {
	var tags []string
	var err error
	m.Range(func(k string, v pcommon.Value) bool {
		if strings.HasPrefix(k, hostTagPrefix) { // User-defined tags
			if str, err2 := assertStringValue(k, v); err2 != nil {
				err = errors.Join(err, err2)
			} else if str == "" {
				err = errors.Join(err, fmt.Errorf("attribute %q has empty string value, expected non-empty string", k))
			} else {
				tags = append(tags, k[len(hostTagPrefix):]+":"+str)
			}
		} else if mappedKey, ok := hostTagMapping[k]; ok { // Well-known tags
			if str, err2 := assertStringValue(k, v); err2 != nil {
				err = errors.Join(err, err2)
			} else {
				tags = append(tags, mappedKey+":"+str)
			}
		}
		return true
	})

	// Allow for comparison of tags
	sort.Strings(tags)
	return tags, err
}

// getHostAliases tries to get a host aliases from attribute datadog.host.aliases
func getHostAliases(attrs pcommon.Map) []string {
	var hostAliases []string
	attrs.Range(func(k string, v pcommon.Value) bool {
		if k == hostAliasAttribute {
			if v.Type() != pcommon.ValueTypeSlice {
				return false
			}
			hostAliasesAny := v.Slice().AsRaw()
			for _, hostAlias := range hostAliasesAny {
				if hostAliasStr, ok := hostAlias.(string); ok {
					hostAliases = append(hostAliases, hostAliasStr)
				}
			}
			return false
		}
		return true
	})
	return hostAliases
}
